@@ -178,6 +178,8 @@ typedef struct _AFS_NONPAGED_FCB
 
     FAST_MUTEX      AdvancedHdrMutex;
 
+    ERESOURCE       CcbListLock;
+
     union
     {
 
@@ -194,6 +196,8 @@ typedef struct _AFS_NONPAGED_FCB
 
             NTSTATUS        ExtentsRequestStatus;
 
+            GUID            ExtentsRequestAuthGroup;
+
             struct _AFS_FSD_EXTENT  *DirtyListHead;
 
             struct _AFS_FSD_EXTENT  *DirtyListTail;
@@ -213,8 +217,6 @@ typedef struct _AFS_NONPAGED_FCB
         struct
         {
 
-            KEVENT          DirectoryEnumEvent;
-
             LONG            DirectoryEnumCount;
 
         } Directory;
@@ -301,10 +303,12 @@ typedef struct AFS_FCB
     struct _AFS_OBJECT_INFORMATION_CB   *ObjectInformation;
 
     //
-    // Authentication group GUID
+    // Ccb list pointers
     //
 
-    GUID  AuthGroup;
+    struct _AFS_CCB    *CcbListHead;
+
+    struct _AFS_CCB    *CcbListTail;
 
     //
     // Union for node type specific information
@@ -372,12 +376,6 @@ typedef struct AFS_FCB
 
             LONG                ExtentCount;
 
-            //
-            // The Lazy writer thread
-            //
-
-            PETHREAD            LazyWriterThread;
-
             //
             // Current count of queued flush items for the file
             //
@@ -424,6 +422,8 @@ typedef struct _AFS_DEVICE_EXTENSION
 
     ULONG           DeviceFlags;
 
+    AFSFcb*         Fcb;
+
     union
     {
 
@@ -550,6 +550,14 @@ typedef struct _AFS_DEVICE_EXTENSION
 
             KEVENT              OutstandingServiceRequestEvent;
 
+            //
+            // Out of memory signalling
+            //
+
+            LONG                WaitingForMemoryCount;
+
+            KEVENT              MemoryAvailableEvent;
+
         } Control;
 
         struct
@@ -584,7 +592,7 @@ typedef struct _AFS_DEVICE_EXTENSION
 
             //
             // Maximum RPC length that is issued by the service. We should limit our
-            // data requests such as for extents to thsi length
+            // data requests such as for extents to this length
             //
 
             ULONG               MaximumRPCLength;
@@ -761,7 +769,7 @@ void *
 
 typedef
 void
-(*PAFSExFreePool)( IN void *Pointer);
+(*PAFSExFreePoolWithTag)( IN void *Pointer, IN ULONG Tag);
 
 typedef
 void
@@ -778,6 +786,8 @@ typedef struct _AFS_LIBRARY_INIT_CB
 
     UNICODE_STRING      AFSServerName;
 
+    UNICODE_STRING      AFSMountRootName;
+
     ULONG               AFSDebugFlags;
 
     AFSFileID           GlobalRootFid;
@@ -800,7 +810,7 @@ typedef struct _AFS_LIBRARY_INIT_CB
 
     PAFSExAllocatePoolWithTag   AFSExAllocatePoolWithTag;
 
-    PAFSExFreePool      AFSExFreePool;
+    PAFSExFreePoolWithTag      AFSExFreePoolWithTag;
 
     PAFSDumpTraceFiles  AFSDumpTraceFiles;
 
