@@ -47,7 +47,7 @@
 //
 // Return:
 //
-//      A status is returned for the function
+//      Return Fcb->NPFcb->Resource held exclusive
 //
 
 NTSTATUS
@@ -74,6 +74,15 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
 
         pVolumeCB = pObjectInfo->VolumeCB;
 
+        if ( pObjectInfo->Fcb != NULL)
+        {
+
+            AFSAcquireExcl( &pObjectInfo->Fcb->NPFcb->Resource,
+                            TRUE);
+
+            try_return( ntStatus = STATUS_SUCCESS);
+        }
+
         //
         // Allocate the Fcb and the nonpaged portion of the Fcb.
         //
@@ -260,8 +269,13 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
             pFcb->Header.NodeTypeCode = AFS_INVALID_FCB;
         }
 
+        pFcb->ObjectInformation = pObjectInfo;
+
+        AFSAcquireShared( &pObjectInfo->NonPagedInfo->ObjectInfoLock,
+                          TRUE);
         //
-        // Initialize some fields in the Fcb
+        // Swap the allocated FCB into the ObjectInformation structure if it
+        // does not already have one.
         //
 
         if ( InterlockedCompareExchangePointer( (PVOID *)&pObjectInfo->Fcb, pFcb, NULL) != NULL)
@@ -280,13 +294,15 @@ AFSInitFcb( IN AFSDirectoryCB  *DirEntry)
                           &pObjectInfo->Fcb->NPFcb->Resource,
                           PsGetCurrentThread());
 
+            AFSReleaseResource( &pObjectInfo->NonPagedInfo->ObjectInfoLock);
+
             AFSAcquireExcl( &pObjectInfo->Fcb->NPFcb->Resource,
                             TRUE);
 
             try_return( ntStatus = STATUS_REPARSE);
         }
 
-        pFcb->ObjectInformation = pObjectInfo;
+        AFSReleaseResource( &pObjectInfo->NonPagedInfo->ObjectInfoLock);
 
         AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                       AFS_TRACE_LEVEL_VERBOSE,
@@ -334,10 +350,10 @@ try_exit:
 
                     ExDeleteResourceLite( &pNPFcb->Resource);
 
-                    AFSExFreePool( pNPFcb);
+                    AFSExFreePoolWithTag( pNPFcb, AFS_FCB_NP_ALLOCATION_TAG);
                 }
 
-                AFSExFreePool( pFcb);
+                AFSExFreePoolWithTag( pFcb, AFS_FCB_ALLOCATION_TAG);
             }
         }
     }
@@ -430,8 +446,6 @@ AFSInitVolume( IN GUID *AuthGroup,
                 AFSAcquireExcl( pVolumeCB->VolumeLock,
                                 TRUE);
 
-                lCount = InterlockedDecrement( &pVolumeCB->VolumeReferenceCount);
-
                 *VolumeCB = pVolumeCB;
 
                 try_return( ntStatus);
@@ -472,7 +486,7 @@ AFSInitVolume( IN GUID *AuthGroup,
 
         pNonPagedVcb = (AFSNonPagedVolumeCB *)AFSExAllocatePoolWithTag( NonPagedPool,
                                                                         sizeof( AFSNonPagedVolumeCB),
-                                                                        AFS_VCB_ALLOCATION_TAG);
+                                                                        AFS_VCB_NP_ALLOCATION_TAG);
 
         if( pNonPagedVcb == NULL)
         {
@@ -493,7 +507,7 @@ AFSInitVolume( IN GUID *AuthGroup,
 
         pNonPagedObject = (AFSNonPagedObjectInfoCB *)AFSExAllocatePoolWithTag( NonPagedPool,
                                                                                sizeof( AFSNonPagedObjectInfoCB),
-                                                                               AFS_VCB_ALLOCATION_TAG);
+                                                                               AFS_NP_OBJECT_INFO_TAG);
 
         if( pNonPagedObject == NULL)
         {
@@ -508,6 +522,8 @@ AFSInitVolume( IN GUID *AuthGroup,
         RtlZeroMemory( pNonPagedObject,
                        sizeof( AFSNonPagedObjectInfoCB));
 
+        ExInitializeResourceLite( &pNonPagedObject->ObjectInfoLock);
+
         ExInitializeResourceLite( &pNonPagedObject->DirectoryNodeHdrLock);
 
         pVolumeCB->NonPagedVcb = pNonPagedVcb;
@@ -526,10 +542,10 @@ AFSInitVolume( IN GUID *AuthGroup,
 
         AFSDbgLogMsg( AFS_SUBSYSTEM_VOLUME_REF_COUNTING,
                       AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSInitVolume Initializing count (1) on volume %08lX\n",
+                      "AFSInitVolume Initializing count (2) on volume %08lX\n",
                       pVolumeCB);
 
-        pVolumeCB->VolumeReferenceCount = 1;
+        pVolumeCB->VolumeReferenceCount = 2;
 
         AFSAcquireExcl( pVolumeCB->VolumeLock,
                         TRUE);
@@ -669,15 +685,15 @@ try_exit:
 
                 ExDeleteResourceLite( &pNonPagedVcb->ObjectInfoTreeLock);
 
-                AFSExFreePool( pNonPagedVcb);
+                AFSExFreePoolWithTag( pNonPagedVcb, AFS_VCB_NP_ALLOCATION_TAG);
             }
 
             if( pNonPagedObject != NULL)
             {
 
-                ExDeleteResourceLite( &pNonPagedObject->DirectoryNodeHdrLock);
+                ExDeleteResourceLite( &pNonPagedObject->ObjectInfoLock);
 
-                AFSExFreePool( pNonPagedObject);
+                AFSExFreePoolWithTag( pNonPagedObject, AFS_NP_OBJECT_INFO_TAG);
             }
 
             if( pVolumeCB != NULL)
@@ -686,10 +702,10 @@ try_exit:
                 if( pVolumeCB->DirectoryCB != NULL)
                 {
 
-                    AFSExFreePool( pVolumeCB->DirectoryCB);
+                    AFSExFreePoolWithTag( pVolumeCB->DirectoryCB, AFS_DIR_ENTRY_TAG);
                 }
 
-                AFSExFreePool( pVolumeCB);
+                AFSExFreePoolWithTag( pVolumeCB, AFS_VCB_ALLOCATION_TAG);
             }
 
             if( pNonPagedDirEntry != NULL)
@@ -697,7 +713,7 @@ try_exit:
 
                 ExDeleteResourceLite( &pNonPagedDirEntry->Lock);
 
-                AFSExFreePool( pNonPagedDirEntry);
+                AFSExFreePoolWithTag( pNonPagedDirEntry, AFS_DIR_ENTRY_NP_TAG);
             }
         }
 
@@ -783,12 +799,21 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
             if( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb != NULL)
             {
 
+                AFSAcquireExcl( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock,
+                                TRUE);
+
                 AFSRemoveFcb( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
+
+                AFSReleaseResource( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock);
             }
 
             AFSDeleteObjectInfo( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
 
-            AFSExFreePool( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB);
+            ExDeleteResourceLite( &VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
+
+            AFSExFreePoolWithTag( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB->NonPaged, AFS_DIR_ENTRY_NP_TAG);
+
+            AFSExFreePoolWithTag( VolumeCB->ObjectInformation.Specific.Directory.PIOCtlDirectoryCB, AFS_DIR_ENTRY_TAG);
         }
 
         if( BooleanFlagOn( VolumeCB->ObjectInformation.Flags, AFS_OBJECT_HELD_IN_SERVICE))
@@ -817,15 +842,17 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
 
             ExDeleteResourceLite( &VolumeCB->NonPagedVcb->ObjectInfoTreeLock);
 
-            AFSExFreePool( VolumeCB->NonPagedVcb);
+            AFSExFreePoolWithTag( VolumeCB->NonPagedVcb, AFS_VCB_NP_ALLOCATION_TAG);
         }
 
         if( VolumeCB->ObjectInformation.NonPagedInfo != NULL)
         {
 
+            ExDeleteResourceLite( &VolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock);
+
             ExDeleteResourceLite( &VolumeCB->ObjectInformation.NonPagedInfo->DirectoryNodeHdrLock);
 
-            AFSExFreePool( VolumeCB->ObjectInformation.NonPagedInfo);
+            AFSExFreePoolWithTag( VolumeCB->ObjectInformation.NonPagedInfo, AFS_NP_OBJECT_INFO_TAG);
         }
 
         if( VolumeCB->DirectoryCB != NULL)
@@ -836,13 +863,13 @@ AFSRemoveVolume( IN AFSVolumeCB *VolumeCB)
 
                 ExDeleteResourceLite( &VolumeCB->DirectoryCB->NonPaged->Lock);
 
-                AFSExFreePool( VolumeCB->DirectoryCB->NonPaged);
+                AFSExFreePoolWithTag( VolumeCB->DirectoryCB->NonPaged, AFS_DIR_ENTRY_NP_TAG);
             }
 
-            AFSExFreePool( VolumeCB->DirectoryCB);
+            AFSExFreePoolWithTag( VolumeCB->DirectoryCB, AFS_DIR_ENTRY_TAG);
         }
 
-        AFSExFreePool( VolumeCB);
+        AFSExFreePoolWithTag( VolumeCB, AFS_VCB_ALLOCATION_TAG);
     }
 
     return ntStatus;
@@ -1012,14 +1039,14 @@ AFSRemoveRootFcb( IN AFSFcb *RootFcb)
         // The non paged region
         //
 
-        AFSExFreePool( RootFcb->NPFcb);
+        AFSExFreePoolWithTag( RootFcb->NPFcb, AFS_FCB_NP_ALLOCATION_TAG);
     }
 
     //
     // And the Fcb itself
     //
 
-    AFSExFreePool( RootFcb);
+    AFSExFreePoolWithTag( RootFcb, AFS_FCB_ALLOCATION_TAG);
 
     return;
 }
@@ -1099,13 +1126,13 @@ AFSRemoveFcb( IN AFSFcb **ppFcb)
     // The non paged region
     //
 
-    AFSExFreePool( pFcb->NPFcb);
+    AFSExFreePoolWithTag( pFcb->NPFcb, AFS_FCB_NP_ALLOCATION_TAG);
 
     //
     // And the Fcb itself, which includes the name
     //
 
-    AFSExFreePool( pFcb);
+    AFSExFreePoolWithTag( pFcb, AFS_FCB_ALLOCATION_TAG);
 
     return;
 }
@@ -1142,8 +1169,25 @@ AFSInitCcb( IN OUT AFSCcb **Ccb)
                        sizeof( AFSCcb));
 
         pCcb->Size = sizeof( AFSCcb);
+
         pCcb->Type = AFS_CCB;
 
+        pCcb->NPCcb = (AFSNonPagedCcb *)AFSExAllocatePoolWithTag( NonPagedPool,
+                                                     sizeof( AFSNonPagedCcb),
+                                                     AFS_CCB_NP_ALLOCATION_TAG);
+
+        if( pCcb->NPCcb == NULL)
+        {
+
+            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
+                          AFS_TRACE_LEVEL_ERROR,
+                          "AFSInitCcb Failed to allocate NPCcb\n");
+
+            try_return( Status = STATUS_INSUFFICIENT_RESOURCES);
+        }
+
+        ExInitializeResourceLite( &pCcb->NPCcb->CcbLock);
+
         //
         // Return the Ccb
         //
@@ -1158,7 +1202,13 @@ try_exit:
             if( pCcb != NULL)
             {
 
-                AFSExFreePool( pCcb);
+                if ( pCcb->NPCcb != NULL)
+                {
+
+                    AFSExFreePoolWithTag( pCcb->NPCcb, AFS_CCB_NP_ALLOCATION_TAG);
+                }
+
+                AFSExFreePoolWithTag( pCcb, AFS_CCB_ALLOCATION_TAG);
             }
 
             *Ccb = NULL;
@@ -1187,6 +1237,9 @@ AFSRemoveCcb( IN AFSFcb *Fcb,
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
 
+    AFSAcquireExcl( &Ccb->NPCcb->CcbLock,
+                    TRUE);
+
     if( Fcb != NULL &&
         BooleanFlagOn( Ccb->Flags, CCB_FLAG_INSERTED_CCB_LIST))
     {
@@ -1230,13 +1283,13 @@ AFSRemoveCcb( IN AFSFcb *Fcb,
     if( Ccb->MaskName.Buffer != NULL)
     {
 
-        AFSExFreePool( Ccb->MaskName.Buffer);
+        AFSExFreePoolWithTag( Ccb->MaskName.Buffer, AFS_GENERIC_MEMORY_6_TAG);
     }
 
     if( BooleanFlagOn( Ccb->Flags, CCB_FLAG_FREE_FULL_PATHNAME))
     {
 
-        AFSExFreePool( Ccb->FullFileName.Buffer);
+        AFSExFreePoolWithTag( Ccb->FullFileName.Buffer, 0);
     }
 
     //
@@ -1254,7 +1307,7 @@ AFSRemoveCcb( IN AFSFcb *Fcb,
     if( Ccb->DirectorySnapshot != NULL)
     {
 
-        AFSExFreePool( Ccb->DirectorySnapshot);
+        AFSExFreePoolWithTag( Ccb->DirectorySnapshot, AFS_DIR_SNAPSHOT_TAG);
 
         Ccb->DirectorySnapshot = NULL;
     }
@@ -1262,14 +1315,20 @@ AFSRemoveCcb( IN AFSFcb *Fcb,
     if( Ccb->NotifyMask.Buffer != NULL)
     {
 
-        AFSExFreePool( Ccb->NotifyMask.Buffer);
+        AFSExFreePoolWithTag( Ccb->NotifyMask.Buffer, AFS_GENERIC_MEMORY_7_TAG);
     }
 
+    AFSReleaseResource( &Ccb->NPCcb->CcbLock);
+
     //
     // Free up the Ccb
     //
 
-    AFSExFreePool( Ccb);
+    ExDeleteResourceLite( &Ccb->NPCcb->CcbLock);
+
+    AFSExFreePoolWithTag( Ccb->NPCcb, AFS_CCB_NP_ALLOCATION_TAG);
+
+    AFSExFreePoolWithTag( Ccb, AFS_CCB_ALLOCATION_TAG);
 
     return ntStatus;
 }
@@ -1284,6 +1343,9 @@ AFSInsertCcb( IN AFSFcb *Fcb,
     AFSAcquireExcl( &Fcb->NPFcb->CcbListLock,
                     TRUE);
 
+    AFSAcquireExcl( &Ccb->NPCcb->CcbLock,
+                    TRUE);
+
     if( Fcb->CcbListHead == NULL)
     {
         Fcb->CcbListHead = Ccb;
@@ -1299,6 +1361,8 @@ AFSInsertCcb( IN AFSFcb *Fcb,
 
     SetFlag( Ccb->Flags, CCB_FLAG_INSERTED_CCB_LIST);
 
+    AFSReleaseResource( &Ccb->NPCcb->CcbLock);
+
     AFSReleaseResource( &Fcb->NPFcb->CcbListLock);
 
     return ntStatus;
