@@ -739,7 +739,8 @@ AFSWorkerThread( IN PVOID Context)
                         {
 
                             AFSReleaseExtentsWithFlush( pWorkItem->Specific.Fcb.Fcb,
-                                                        &pWorkItem->AuthGroup);
+                                                        &pWorkItem->AuthGroup,
+                                                        FALSE);
                         }
 
                         ASSERT( pWorkItem->Specific.Fcb.Fcb->OpenReferenceCount != 0);
@@ -749,29 +750,6 @@ AFSWorkerThread( IN PVOID Context)
                         break;
                     }
 
-                    case AFS_WORK_ASYNCH_READ:
-                    {
-
-                        ASSERT( pWorkItem->Specific.AsynchIo.CallingProcess != NULL);
-
-                        (VOID) AFSCommonRead( pWorkItem->Specific.AsynchIo.Device,
-                                              pWorkItem->Specific.AsynchIo.Irp,
-                                              pWorkItem->Specific.AsynchIo.CallingProcess);
-
-                        break;
-                    }
-
-                    case AFS_WORK_ASYNCH_WRITE:
-                    {
-
-                        ASSERT( pWorkItem->Specific.AsynchIo.CallingProcess != NULL);
-
-                        (VOID) AFSCommonWrite( pWorkItem->Specific.AsynchIo.Device,
-                                               pWorkItem->Specific.AsynchIo.Irp,
-                                               pWorkItem->Specific.AsynchIo.CallingProcess);
-                        break;
-                    }
-
                     case AFS_WORK_ENUMERATE_GLOBAL_ROOT:
                     {
 
@@ -986,6 +964,7 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
     LONG lFileType;
     LARGE_INTEGER liCurrentTime;
     BOOLEAN bVolumeObject = FALSE;
+    BOOLEAN bFcbBusy = FALSE;
     LONG lCount;
 
     pControlDeviceExt = (AFSDeviceExt *)AFSControlDeviceObject->DeviceExtension;
@@ -1029,11 +1008,20 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
     while( BooleanFlagOn( pPoolContext->State, AFS_WORKER_PROCESS_REQUESTS))
     {
 
-        KeWaitForSingleObject( &Timer,
-                               Executive,
-                               KernelMode,
-                               FALSE,
-                               NULL);
+        if ( bFcbBusy == FALSE)
+        {
+
+            KeWaitForSingleObject( &Timer,
+                                   Executive,
+                                   KernelMode,
+                                   FALSE,
+                                   NULL);
+        }
+        else
+        {
+
+            bFcbBusy = FALSE;
+        }
 
         //
         // This is the primary volume worker so it will traverse the volume list
@@ -1088,6 +1076,9 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                 pNextVolume = (AFSVolumeCB *)pVolumeCB->ListEntry.fLink;
 
+                AFSAcquireShared( &pVolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock,
+                                  TRUE);
+
                 if( pVolumeCB->ObjectInfoListHead == NULL &&
                     pVolumeCB->DirectoryCB->OpenReferenceCount == 0 &&
                     pVolumeCB->VolumeReferenceCount == 1 &&
@@ -1102,11 +1093,15 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                         AFSRemoveRootFcb( pVolumeCB->RootFcb);
                     }
 
+                    AFSReleaseResource( &pVolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock);
+
                     AFSRemoveVolume( pVolumeCB);
                 }
                 else
                 {
 
+                    AFSReleaseResource( &pVolumeCB->ObjectInformation.NonPagedInfo->ObjectInfoLock);
+
                     AFSReleaseResource( pVolumeCB->VolumeLock);
                 }
 
@@ -1193,33 +1188,38 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                                 FALSE))
                             {
 
-                                if ( pCurrentObject->ObjectReferenceCount <= 0)
+                                AFSAcquireExcl( &pCurrentObject->NonPagedInfo->ObjectInfoLock,
+                                                TRUE);
+
+                                if ( pCurrentObject->ObjectReferenceCount <= 0 &&
+                                     ( pCurrentObject->Fcb == NULL ||
+                                       pCurrentObject->Fcb->OpenReferenceCount == 0 &&
+                                       pCurrentObject->Fcb->Specific.File.ExtentCount == 0))
                                 {
 
-                                    if( pCurrentObject->Fcb != NULL)
-                                    {
-
-                                        AFSRemoveFcb( &pCurrentObject->Fcb);
-                                    }
+                                    AFSRemoveFcb( &pCurrentObject->Fcb);
 
                                     if( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB != NULL)
                                     {
 
-                                        if( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb != NULL)
-                                        {
+                                        AFSAcquireExcl( &pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock,
+                                                        TRUE);
 
-                                            AFSRemoveFcb( &pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
-                                        }
+                                        AFSRemoveFcb( &pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
+
+                                        AFSReleaseResource( &pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock);
 
                                         AFSDeleteObjectInfo( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
 
                                         ExDeleteResourceLite( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
 
-                                        AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
+                                        AFSExFreePoolWithTag( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged, AFS_DIR_ENTRY_NP_TAG);
 
-                                        AFSExFreePool( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB);
+                                        AFSExFreePoolWithTag( pCurrentObject->Specific.Directory.PIOCtlDirectoryCB, AFS_DIR_ENTRY_TAG);
                                     }
 
+                                    AFSReleaseResource( &pCurrentObject->NonPagedInfo->ObjectInfoLock);
+
                                     AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                                   AFS_TRACE_LEVEL_VERBOSE,
                                                   "AFSPrimaryVolumeWorkerThread Deleting deleted object %08lX\n",
@@ -1227,6 +1227,11 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                     AFSDeleteObjectInfo( pCurrentObject);
                                 }
+                                else
+                                {
+
+                                    AFSReleaseResource( &pCurrentObject->NonPagedInfo->ObjectInfoLock);
+                                }
 
                                 AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
 
@@ -1387,7 +1392,15 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                     AFSDeleteDirEntry( pCurrentObject,
                                                        pCurrentDirEntry);
 
-                                    if( pCurrentChildObject->ObjectReferenceCount <= 0 &&
+
+                                    //
+                                    // Acquire ObjectInfoLock shared here so as not to deadlock
+                                    // with an invalidation call from the service during AFSCleanupFcb
+                                    //
+
+                                    lCount = AFSObjectInfoIncrement( pCurrentChildObject);
+
+                                    if( lCount == 1 &&
                                         pCurrentChildObject->Fcb != NULL &&
                                         pCurrentChildObject->FileType == AFS_FILE_TYPE_FILE)
                                     {
@@ -1400,46 +1413,63 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                         AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
 
+                                        //
+                                        // Cannot hold a TreeLock across an AFSCleanupFcb call
+                                        // as it can deadlock with an invalidation ioctl initiated
+                                        // from the service.
                                         //
                                         // Dropping the TreeLock permits the
                                         // pCurrentObject->ObjectReferenceCount to change
                                         //
 
-                                        AFSCleanupFcb( pCurrentChildObject->Fcb,
-                                                       TRUE);
+                                        ntStatus = AFSCleanupFcb( pCurrentChildObject->Fcb,
+                                                                  TRUE);
+
+                                        if ( ntStatus == STATUS_RETRY)
+                                        {
+
+                                            bFcbBusy = TRUE;
+                                        }
 
                                         AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
                                                         TRUE);
                                     }
 
-                                    if( pCurrentChildObject->ObjectReferenceCount <= 0)
+                                    lCount = AFSObjectInfoDecrement( pCurrentChildObject);
+
+                                    AFSAcquireExcl( &pCurrentChildObject->NonPagedInfo->ObjectInfoLock,
+                                                    TRUE);
+
+                                    if( pCurrentChildObject->ObjectReferenceCount <= 0 &&
+                                        ( pCurrentChildObject->Fcb == NULL ||
+                                          pCurrentChildObject->Fcb->OpenReferenceCount == 0 &&
+                                          pCurrentChildObject->Fcb->Specific.File.ExtentCount == 0))
                                     {
 
-                                        if( pCurrentChildObject->Fcb != NULL)
-                                        {
-                                        
-                                            AFSRemoveFcb( &pCurrentChildObject->Fcb);
-                                        }
+                                        AFSRemoveFcb( &pCurrentChildObject->Fcb);
 
                                         if( pCurrentChildObject->FileType == AFS_FILE_TYPE_DIRECTORY &&
                                             pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB != NULL)
                                         {
 
-                                            if( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb != NULL)
-                                            {
+                                            AFSAcquireExcl( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock,
+                                                            TRUE);
 
-                                                AFSRemoveFcb( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
-                                            }
+                                            AFSRemoveFcb( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->Fcb);
+
+                                            AFSReleaseResource( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation->NonPagedInfo->ObjectInfoLock);
 
                                             AFSDeleteObjectInfo( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->ObjectInformation);
 
                                             ExDeleteResourceLite( &pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged->Lock);
 
-                                            AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged);
+                                            AFSExFreePoolWithTag( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB->NonPaged, AFS_DIR_ENTRY_NP_TAG);
 
-                                            AFSExFreePool( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB);
+                                            AFSExFreePoolWithTag( pCurrentChildObject->Specific.Directory.PIOCtlDirectoryCB, AFS_DIR_ENTRY_TAG);
                                         }
 
+                                        AFSReleaseResource( &pCurrentChildObject->NonPagedInfo->ObjectInfoLock);
+
                                         AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
                                                       AFS_TRACE_LEVEL_VERBOSE,
                                                       "AFSPrimaryVolumeWorkerThread Deleting object %08lX\n",
@@ -1447,6 +1477,11 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                         AFSDeleteObjectInfo( pCurrentChildObject);
                                     }
+                                    else
+                                    {
+
+                                        AFSReleaseResource( &pCurrentChildObject->NonPagedInfo->ObjectInfoLock);
+                                    }
 
                                     pCurrentDirEntry = pNextDirEntry;
 
@@ -1534,7 +1569,10 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                         AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
 
-                        if( pCurrentObject->Fcb != NULL)
+                        lCount = AFSObjectInfoIncrement( pCurrentObject);
+
+                        if( lCount == 0 &&
+                            pCurrentObject->Fcb != NULL)
                         {
 
                             //
@@ -1542,10 +1580,18 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                             // pCurrentObject->ObjectReferenceCount to change
                             //
 
-                            AFSCleanupFcb( pCurrentObject->Fcb,
-                                           TRUE);
+                            ntStatus = AFSCleanupFcb( pCurrentObject->Fcb,
+                                                      FALSE);
+
+                            if ( ntStatus == STATUS_RETRY)
+                            {
+
+                                bFcbBusy = TRUE;
+                            }
                         }
 
+                        lCount = AFSObjectInfoDecrement( pCurrentObject);
+
                         if( !AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
                                              FALSE))
                         {
@@ -1555,20 +1601,27 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                             break;
                         }
 
+                        AFSAcquireExcl( &pCurrentObject->NonPagedInfo->ObjectInfoLock,
+                                        TRUE);
+
                         if( BooleanFlagOn( pCurrentObject->Flags, AFS_OBJECT_FLAGS_DELETED) &&
                             pCurrentObject->ObjectReferenceCount <= 0 &&
                             ( pCurrentObject->Fcb == NULL ||
-                              pCurrentObject->Fcb->OpenReferenceCount == 0))
+                              pCurrentObject->Fcb->OpenReferenceCount == 0 &&
+                              pCurrentObject->Fcb->Specific.File.ExtentCount == 0))
                         {
 
-                            if( pCurrentObject->Fcb != NULL)
-                            {
+                            AFSRemoveFcb( &pCurrentObject->Fcb);
 
-                                AFSRemoveFcb( &pCurrentObject->Fcb);
-                            }
+                            AFSReleaseResource( &pCurrentObject->NonPagedInfo->ObjectInfoLock);
 
                             AFSDeleteObjectInfo( pCurrentObject);
                         }
+                        else
+                        {
+
+                            AFSReleaseResource( &pCurrentObject->NonPagedInfo->ObjectInfoLock);
+                        }
 
                         AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
 
@@ -2213,6 +2266,8 @@ try_exit:
 
         lCount = InterlockedDecrement( &Fcb->Specific.File.QueuedFlushCount);
 
+	ASSERT( lCount >= 0);
+
         if( lCount == 0)
         {
 
@@ -2250,178 +2305,6 @@ try_exit:
     return ntStatus;
 }
 
-NTSTATUS
-AFSQueueAsyncRead( IN PDEVICE_OBJECT DeviceObject,
-                   IN PIRP Irp,
-                   IN HANDLE CallerProcess)
-{
-
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    AFSWorkItem *pWorkItem = NULL;
-
-    __try
-    {
-
-        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
-                      AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSQueueAsyncRead Queuing request for Irp %08lX\n",
-                      Irp);
-
-        pWorkItem = (AFSWorkItem *) AFSLibExAllocatePoolWithTag( NonPagedPool,
-                                                                 sizeof(AFSWorkItem),
-                                                                 AFS_WORK_ITEM_TAG);
-        if (NULL == pWorkItem)
-        {
-
-            AFSDbgLogMsg( AFS_SUBSYSTEM_IO_PROCESSING,
-                          AFS_TRACE_LEVEL_ERROR,
-                          "AFSQueueAsyncRead Failed to allocate work item\n");
-
-            try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES );
-        }
-
-        RtlZeroMemory( pWorkItem,
-                       sizeof(AFSWorkItem));
-
-        pWorkItem->Size = sizeof( AFSWorkItem);
-
-        pWorkItem->RequestType = AFS_WORK_ASYNCH_READ;
-
-        pWorkItem->Specific.AsynchIo.Device = DeviceObject;
-
-        pWorkItem->Specific.AsynchIo.Irp = Irp;
-
-        pWorkItem->Specific.AsynchIo.CallingProcess = CallerProcess;
-
-        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
-                      AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSQueueAsyncRead Workitem %08lX for Irp %08lX\n",
-                      pWorkItem,
-                      Irp);
-
-        ntStatus = AFSQueueWorkerRequest( pWorkItem);
-
-try_exit:
-
-        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
-                      AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSQueueAsyncRead Request for Irp %08lX complete Status %08lX\n",
-                      Irp,
-                      ntStatus);
-
-        if( !NT_SUCCESS( ntStatus))
-        {
-
-            if( pWorkItem != NULL)
-            {
-
-                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
-            }
-
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
-                          AFS_TRACE_LEVEL_ERROR,
-                          "AFSQueueAsyncRead Failed to queue request Status %08lX\n", ntStatus);
-        }
-    }
-    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
-    {
-
-        AFSDbgLogMsg( 0,
-                      0,
-                      "EXCEPTION - AFSQueueAsyncRead\n");
-
-        AFSDumpTraceFilesFnc();
-    }
-
-    return ntStatus;
-}
-
-NTSTATUS
-AFSQueueAsyncWrite( IN PDEVICE_OBJECT DeviceObject,
-                    IN PIRP Irp,
-                    IN HANDLE CallerProcess)
-{
-
-    NTSTATUS ntStatus = STATUS_SUCCESS;
-    AFSWorkItem *pWorkItem = NULL;
-
-    __try
-    {
-
-        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
-                      AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSQueueAsyncWrite Queuing request for Irp %08lX\n",
-                      Irp);
-
-        pWorkItem = (AFSWorkItem *) AFSLibExAllocatePoolWithTag( NonPagedPool,
-                                                                 sizeof(AFSWorkItem),
-                                                                 AFS_WORK_ITEM_TAG);
-        if (NULL == pWorkItem)
-        {
-
-            AFSDbgLogMsg( AFS_SUBSYSTEM_IO_PROCESSING,
-                          AFS_TRACE_LEVEL_ERROR,
-                          "AFSQueueAsyncWrite Failed to allocate work item\n");
-
-            try_return( ntStatus = STATUS_INSUFFICIENT_RESOURCES );
-        }
-
-        RtlZeroMemory( pWorkItem,
-                       sizeof(AFSWorkItem));
-
-        pWorkItem->Size = sizeof( AFSWorkItem);
-
-        pWorkItem->RequestType = AFS_WORK_ASYNCH_WRITE;
-
-        pWorkItem->Specific.AsynchIo.Device = DeviceObject;
-
-        pWorkItem->Specific.AsynchIo.Irp = Irp;
-
-        pWorkItem->Specific.AsynchIo.CallingProcess = CallerProcess;
-
-        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
-                      AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSQueueAsyncWrite Workitem %08lX for Irp %08lX\n",
-                      pWorkItem,
-                      Irp);
-
-        ntStatus = AFSQueueWorkerRequest( pWorkItem);
-
-try_exit:
-
-        AFSDbgLogMsg( AFS_SUBSYSTEM_WORKER_PROCESSING,
-                      AFS_TRACE_LEVEL_VERBOSE,
-                      "AFSQueueAsyncWrite Request for Irp %08lX complete Status %08lX\n",
-                      Irp,
-                      ntStatus);
-
-        if( !NT_SUCCESS( ntStatus))
-        {
-
-            if( pWorkItem != NULL)
-            {
-
-                ExFreePoolWithTag( pWorkItem, AFS_WORK_ITEM_TAG);
-            }
-
-            AFSDbgLogMsg( AFS_SUBSYSTEM_FILE_PROCESSING,
-                          AFS_TRACE_LEVEL_ERROR,
-                          "AFSQueueAsyncWrite Failed to queue request Status %08lX\n", ntStatus);
-        }
-    }
-    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
-    {
-
-        AFSDbgLogMsg( 0,
-                      0,
-                      "EXCEPTION - AFSQueueAsyncWrite\n");
-
-        AFSDumpTraceFilesFnc();
-    }
-
-    return ntStatus;
-}
-
 NTSTATUS
 AFSQueueGlobalRootEnumeration()
 {
