@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -11,22 +11,16 @@
  * Description:
  *	Implementation of the gator text object.
  *------------------------------------------------------------------------*/
-#define	IGNORE_STDS_H
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/gtx/textobject.c,v 1.6.14.1 2007/10/30 15:16:39 shadow Exp $");
+#include <roken.h>
 
 #include "gtxtextobj.h"		/*Interface for this module */
 #include "gtxwindows.h"		/*Gator window interface */
 #include "gtxcurseswin.h"	/*Gator curses window interface */
 #include "gtxdumbwin.h"		/*Gator dumb terminal window interface */
 #include "gtxX11win.h"		/*Gator X11 window interface */
-#include <stdio.h>		/*Standard I/O stuff */
-#include <errno.h>
-#include <string.h>
-#include <stdlib.h>
 
 /*Externally-advertised array of text onode operations*/
 struct onodeops gator_text_ops = {
@@ -66,10 +60,7 @@ static char mn[] = "gator_textobject";	/*Module name */
  *------------------------------------------------------------------------*/
 
 int
-gator_text_create(text_onp, params)
-     struct onode *text_onp;
-     struct onode_createparams *params;
-
+gator_text_create(struct onode *text_onp, struct onode_createparams *params)
 {				/*gator_text_create */
 
     static char rn[] = "gator_text_create";	/*Routine name */
@@ -80,7 +71,7 @@ gator_text_create(text_onp, params)
     text_params = (struct gator_textobj_params *)params;
     if (objects_debug) {
 	fprintf(stderr,
-		"[%s:%s] Private data passed to text object at 0x%x:\n", mn,
+		"[%s:%s] Private data passed to text object at %p:\n", mn,
 		rn, text_onp);
 	fprintf(stderr, "\tmaxEntries: %d, maxCharsPerEntry: %d\n",
 		text_params->maxEntries, text_params->maxCharsPerEntry);
@@ -91,12 +82,12 @@ gator_text_create(text_onp, params)
      */
     if (objects_debug)
 	fprintf(stderr,
-		"[%s:%s] Allocating %d bytes for text object private data region\n",
+		"[%s:%s] Allocating %" AFS_SIZET_FMT " bytes for text object private data region\n",
 		mn, rn, sizeof(struct gator_textobj));
     text_data = (struct gator_textobj *)malloc(sizeof(struct gator_textobj));
     if (text_data == (struct gator_textobj *)0) {
 	fprintf(stderr,
-		"[%s:%s] Can't allocate %d bytes for text object private data region, errno is %d\n",
+		"[%s:%s] Can't allocate %" AFS_SIZET_FMT " bytes for text object private data region, errno is %d\n",
 		mn, rn, sizeof(struct gator_textobj), errno);
 	return (errno);
     }
@@ -159,9 +150,7 @@ gator_text_create(text_onp, params)
  *------------------------------------------------------------------------*/
 
 int
-gator_text_destroy(onp)
-     struct onode *onp;
-
+gator_text_destroy(struct onode *onp)
 {				/*gator_text_destroy */
 
     /*
@@ -192,9 +181,7 @@ gator_text_destroy(onp)
  *------------------------------------------------------------------------*/
 
 int
-gator_text_display(onp)
-     struct onode *onp;
-
+gator_text_display(struct onode *onp)
 {				/*gator_text_display */
 
     static char rn[] = "gator_text_display";	/*Routine name */
@@ -208,13 +195,13 @@ gator_text_display(onp)
     struct gator_textcb_entry *curr_ent;	/*Ptr to current entry */
 
     if (objects_debug)
-	fprintf(stderr, "[%s:%s] Displaying text object at 0x%x\n", mn, rn,
+	fprintf(stderr, "[%s:%s] Displaying text object at %p\n", mn, rn,
 		onp);
     text_data = (struct gator_textobj *)(onp->o_data);
     cbHdr = text_data->cbHdr;
     if (objects_debug)
 	fprintf(stderr,
-		"[%s:%s] Displaying text object at 0x%x, object-specific data at 0x%x\n",
+		"[%s:%s] Displaying text object at %p, object-specific data at %p\n",
 		mn, rn, onp, text_data);
 
     /*
@@ -229,7 +216,7 @@ gator_text_display(onp)
 
     if (objects_debug)
 	fprintf(stderr,
-		"[%s:%s] Drawing %d populated lines, starting with entry %d (index %d) at 0x%x\n",
+		"[%s:%s] Drawing %d populated lines, starting with entry %d (index %d) at %p",
 		mn, rn, currLinesUsed, currEnt, currIdx, curr_ent);
 
     strparams.x = onp->o_x;
@@ -311,9 +298,7 @@ gator_text_display(onp)
  *------------------------------------------------------------------------*/
 
 int
-gator_text_release(onp)
-     struct onode *onp;
-
+gator_text_release(struct onode *onp)
 {				/*gator_text_release */
 
     /*
@@ -347,11 +332,7 @@ gator_text_release(onp)
  *------------------------------------------------------------------------*/
 
 int
-gator_text_Scroll(onp, nlines, direction)
-     struct onode *onp;
-     int nlines;
-     int direction;
-
+gator_text_Scroll(struct onode *onp, int nlines, int direction)
 {				/*gator_text_Scroll */
 
     static char rn[] = "gator_text_Scroll";	/*Routine name */
@@ -363,7 +344,7 @@ gator_text_Scroll(onp, nlines, direction)
      * the window empty.
      */
     if (objects_debug)
-	fprintf(stderr, "[%s:%s] Scrolling text object at 0x%x %d lines %s\n",
+	fprintf(stderr, "[%s:%s] Scrolling text object %d lines %s\n",
 		mn, rn, nlines,
 		(direction == GATOR_TEXT_SCROLL_UP) ? "UP" : "DOWN");
 
@@ -428,17 +409,12 @@ gator_text_Scroll(onp, nlines, direction)
  *------------------------------------------------------------------------*/
 
 int
-gator_text_Write(onp, strToWrite, numChars, highlight, skip)
-     struct onode *onp;
-     char *strToWrite;
-     int numChars;
-     int highlight;
-     int skip;
-
+gator_text_Write(struct onode *onp, char *strToWrite, int numChars,
+		 int highlight, int skip)
 {				/*gator_text_Write */
 
     static char rn[] = "gator_text_Write";	/*Routine name */
-    register int code;		/*Return value on routines */
+    int code;		/*Return value on routines */
     struct gator_textobj *text_data;	/*Ptr to text obj data */
     struct gator_textcb_hdr *cbHdr;	/*Ptr to text CB header */
     int i;			/*Loop variable */
@@ -449,14 +425,14 @@ gator_text_Write(onp, strToWrite, numChars, highlight, skip)
     int bumpAmount;		/*Amount to bump count */
 
     /*
-     * 
+     *
      */
     if (objects_debug) {
 	fprintf(stderr,
-		"[%s:%s] Writing %d chars to text object at 0x%x (highlight=%d, skip=%d: '",
-		rn, numChars, onp, highlight, skip);
+		"[%s:%s] Writing %d chars to text object at %p (highlight=%d, skip=%d: '",
+		mn, rn, numChars, onp, highlight, skip);
 	for (i = 0; i < numChars; i++)
-	    fprintf(stderr, "%c", strToWrite + i);
+	    fprintf(stderr, "%c", strToWrite[i]);
 	fprintf(stderr, "\n");
     }
 
@@ -574,14 +550,11 @@ gator_text_Write(onp, strToWrite, numChars, highlight, skip)
  *------------------------------------------------------------------------*/
 
 int
-gator_text_BlankLine(onp, numBlanks)
-     struct onode *onp;
-     int numBlanks;
-
+gator_text_BlankLine(struct onode *onp, int numBlanks)
 {				/*gator_text_BlankLine */
 
     static char rn[] = "gator_text_BlankLine";	/*Routine name */
-    register int code;		/*Return value on routines */
+    int code;		/*Return value on routines */
     struct gator_textobj *text_data;	/*Ptr to text obj data */
 
     /*
@@ -589,14 +562,14 @@ gator_text_BlankLine(onp, numBlanks)
      */
     if (objects_debug)
 	fprintf(stderr,
-		"[%s:%s] Writing %d blank lines to text object at 0x%x\n", mn,
+		"[%s:%s] Writing %d blank lines to text object at %p\n", mn,
 		rn, numBlanks, onp);
 
     text_data = (struct gator_textobj *)(onp->o_data);
     code = gator_textcb_BlankLine(text_data->cbHdr, numBlanks);
     if (code) {
 	fprintf(stderr,
-		"[%s:%s] Can't write %d blank lines to text object at 0x%x\n",
+		"[%s:%s] Can't write %d blank lines to text object at %p\n",
 		mn, rn, numBlanks, onp);
 	return (code);
     }
